#include <gidx.h>
#include <fobuf.h>
#include <gidx-format.h>
#include <gidx-wr.h>
#define _THIS_IS_LIBGIDX_WR 1
#include "gidx-wr-core.h"
#define _THIS_IS_GIDX_WR_C 1
#include "gidx-pkey.h"

static gidx_pkey_t pbuf_max;
static gidx_pkey_t pbuf_sz;
static gidx_pkey_t *pbuf;

int pkey_init(pkey_ctx_t ctx, const gidx_wr_t wr, fobuf_t buf)
{
	ctx->ctx_buf = buf;
	ctx->ctx_tot_rec = gidx_wr_num_pkey(wr);
	ctx->ctx_num_rec = 0;
	ctx->ctx_num_val = 0;
	ctx->ctx_cur_val = 0;
	return 1;
}

int pkey_tbl_entry(pkey_ctx_t ctx, gidx_pkey_t num_pkey)
{
	struct gidx_pkey_tbl tbl;

	tbl.num_pkey = num_pkey;
	tbl.pkey_idx = ctx->ctx_num_rec;
	if ( !fobuf_write(ctx->ctx_buf, &tbl, sizeof(tbl)) )
		return 0;

	ctx->ctx_num_rec += num_pkey;
	ctx->ctx_num_val++;

	if ( num_pkey > pbuf_max )
		pbuf_max = num_pkey;
	return 1;
}

int pkey_init2(pkey_ctx_t ctx)
{
	if ( ctx->ctx_tot_rec < ctx->ctx_num_rec ) {
		ERR("tot_rec/num_rec mismatch %u/%u",
			ctx->ctx_tot_rec, ctx->ctx_num_rec);
		return 0;
	}

	if ( pbuf_max > pbuf_sz ) {
		gidx_pkey_t *new;

		new = realloc(pbuf, sizeof(*pbuf) * pbuf_max);
		if ( NULL == new ) {
			ERR("realloc: %s", os_err());
			return 0;
		}

		pbuf = new;
		pbuf_sz = pbuf_max;
	}

	return 1;
}

static int pkey_cb(void *priv, void *obj)
{
	gidx_pkey_t **pb = priv;
	gidx_pkey_t *key = obj;

	if ( (*pb) > (pbuf + pbuf_sz) )
		return 0;

	(**pb) = (*key);
	(*pb)++;
	return 1;
}

int pkey_keyblock(pkey_ctx_t ctx, hgang_t pkeys)
{
	gidx_pkey_t *pb_tmp;
	gidx_pkey_t num_pkey;

	pb_tmp = pbuf;

	if ( !hgang_foreach(pkeys, pkey_cb, &pb_tmp) ) {
		ERR("pkey_block larger than expected");
		return 0;
	}

	num_pkey = pb_tmp - pbuf;

	sort(pbuf, num_pkey, sizeof(*pbuf), gidx_pkey_cmp);
	ctx->ctx_cur_val++;

	if ( !fobuf_write(ctx->ctx_buf, pbuf, sizeof(*pbuf) * num_pkey) )
		return 0;

	return 1;
}

int pkey_fini(pkey_ctx_t ctx)
{
	if ( ctx->ctx_cur_val != ctx->ctx_num_val ) {
		ERR("cur_val/num_val mismatch %u/%u",
			ctx->ctx_cur_val, ctx->ctx_num_val);
		return 0;
	}
	if ( ctx->ctx_num_rec > ctx->ctx_tot_rec ) {
		ERR("num_rec/tot_rec mismatch %u/%u",
			ctx->ctx_num_rec, ctx->ctx_tot_rec);
		return 0;
	}
	return 1;
}

