/*
* This file is part of blackbloc
* Copyright (c) 2010 Gianni Tedesco
* Released under the terms of the GNU GPL version 3
*/
#ifndef _GIDX_FORMAT_H
#define _GIDX_FORMAT_H

#if _FILE_OFFSET_BITS == 64
typedef uint64_t gidx_off_t;
#else
typedef uint32_t gidx_off_t;
#endif

#define GIDX_MAGIC 		0xdad5babe
#define GIDX_MAX_TYPE		0xffU
#define GIDX_MAX_INDEX		0xffU
#define GIDX_MAX_INDEX_NLEN	0xffU
#define GIDX_VERSION		0x00000001
#define GIDX_ALL_FEATURES	0

#define GIDX_CHUNK_USER		0

#define GIDX_CHUNK_PKEY_TBL	0xfe
#define GIDX_CHUNK_PKEY_BLOCK	0xff

/* Align all records to 8 bytes */
#define GIDX_ALIGN_BITS 3ULL
#define GIDX_ALIGN (1ULL << GIDX_ALIGN_BITS)
#define GIDX_ALIGN_MASK (GIDX_ALIGN - 1ULL)
#define GIDX_LEN_MASK (~GIDX_ALIGN_MASK)
#define ALIGN_GIDX(x) ((x + GIDX_ALIGN_MASK) & GIDX_LEN_MASK)
#define GIDX_ALIGN_BYTES(x) ((GIDX_ALIGN - (x & GIDX_ALIGN_MASK)) \
				& GIDX_ALIGN_MASK)

struct gidx_hdr {
	/** Magic */
	uint32_t h_magic;
	uint16_t h_version;
	uint16_t h_features;
	/** Total number of chunks for all indexes */
	uint32_t h_num_chunks;
	/* Number of indexe descriptors in index block */
	uint16_t h_num_idx;
	uint8_t _pad0;
	uint8_t h_strtab_sz_hi;
	/** Size of string table for index names */
	uint16_t h_strtab_sz_lo;
	uint16_t _pad1;
	/* FIXME: */
	//uint32_t _align;
}_packed;

struct gidx_idx {
	/** Index in to chunk block of first chunk */
	uint32_t i_chunk_idx;
	/** Number of data chunks associated with this index */
	uint16_t i_num_chunks;
	/** Length of the index-name. */
	uint8_t i_nlen;
	/** Type of index */
	uint8_t i_type;
	/** primary/secondary/non key */
	uint8_t i_mode;
	/** Format of index */
	uint8_t i_format;
	uint8_t _pad0;
	/** Offset of filename in to strtab block. */
	uint8_t i_name_hi;
	uint16_t i_name_lo;
	uint16_t i_sys_chunks;
}_packed;

struct gidx_chunk {
	/** offset in to data block for this chunk */
	gidx_off_t c_data_ofs;
	/** length of this chunks data */
	uint32_t c_data_len;
	/** Chunk data format */
	uint8_t c_format;
	uint8_t _pad0;
	uint16_t _pad1;
}_packed;

struct gidx_pkey_tbl {
	gidx_pkey_t num_pkey;
	gidx_pkey_t pkey_idx;
}_packed;


#endif /* _GIDX_FORMAT_H */
