/*
* This file is part of blackbloc
* Copyright (c) 2010 Gianni Tedesco
* Released under the terms of the GNU GPL version 3
*/
#ifndef _GIDX_WR_H
#define _GIDX_WR_H

typedef struct _gidx_wr *gidx_wr_t;

#define GIDX_STRING_CASE_SENSITIVE	(1<<0)
/** Keys are to be ordered lexicographically */
#define GIDX_STRING_LEX			(1<<1)
/** enable suffix queries */
#define GIDX_STRING_SUFFIX		(1<<2)
/** enable prefix queries */
#define GIDX_STRING_PREFIX		(1<<3)
/** Enable compression based on the alphabet of symbols which appear in keys */
#define GIDX_STRING_COMPRESS_ALPHABET	(1<<4)

typedef int (*gidx_get_str_t)(void *record, void *field, struct ro_vec *vec);
typedef int (*gidx_get_uint_t)(void *record, void *field, uint32_t *val);
typedef int (*gidx_get_interval_t)(void *record, void *field,
					uint32_t *begin,
					uint32_t *end);

union schema_get_val {
	gidx_get_str_t		su_get_str;
	gidx_get_uint_t		su_get_uint;
	gidx_get_interval_t	su_get_interval;
};

struct gidx_schema {
	const char 		*s_name;
	union schema_get_val 	s_get_val;
	gidx_type_t		s_type;
	gidx_mode_t		s_mode;
	unsigned int 		s_options;
};

_public _check_result gidx_wr_t gidx_wr_new(const struct gidx_schema *schema,
						unsigned int num);
_public void gidx_wr_record_init(gidx_wr_t wr, void *record);
_public void gidx_wr_record_set(gidx_wr_t, unsigned int field, void *priv);
_public _check_result int gidx_wr_record_commit(gidx_wr_t wr);
_public void gidx_wr_record_abort(gidx_wr_t wr);
_public _check_result int gidx_wr_write(gidx_wr_t wr, int fd);
_public void gidx_wr_free(gidx_wr_t wr);

#endif /* _GIDX_WR_H */
