/*
 * This file is part of Firestorm NIDS.
 * Copyright (c) 2008 Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL version 3
*/
#ifndef _GIDX_H
#define _GIDX_H

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <compiler.h>

/* Standard C99 stuff */
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#if HAVE_ASSERT_H
#include <assert.h>
#endif

#include <os.h>
#include <list.h>

struct vec {
	uint8_t *v_ptr;
	size_t v_len;
};

struct ro_vec {
	const uint8_t *v_ptr;
	size_t v_len;
};

typedef uint8_t gidx_type_t;
typedef uint8_t gidx_mode_t;

#if GIDX_64BIT_PKEY
typedef uint64_t gidx_pkey_t;
#else
typedef uint32_t gidx_pkey_t;
#endif

#define DEBUG(fmt, x...) mesg(M_DEBUG, "%s:%u: " fmt, __FUNCTION__, __LINE__, ##x)
#define INFO(fmt, x...) mesg(M_INFO, "%s:%u: " fmt, __FUNCTION__, __LINE__, ##x)
#define WARN(fmt, x...) mesg(M_WARN, "%s:%u: " fmt, __FUNCTION__, __LINE__, ##x)
#define ERR(fmt, x...) mesg(M_ERR, "%s:%u: " fmt, __FUNCTION__, __LINE__, ##x)
#define CRIT(fmt, x...) mesg(M_CRIT, "%s:%u: " fmt, __FUNCTION__, __LINE__, ##x)

/* Message codes */
#define M_UNSET	0 /* Unclassified */
#define M_DEBUG	1 /* For developers */
#define M_INFO	2 /* Informational notices */
#define M_WARN	3 /* We can work around this, but you should know */
#define M_ERR	4 /* We can't do something you asked for */
#define M_CRIT	5 /* Service/data is lost */
#define M_MAX	6
#define M_LIMIT 0x80 /* Ratelimit this message */

/* --- Shared gidx read/write stuff */
typedef uint8_t mesg_code_t;
_protected void mesg(mesg_code_t code, const char *fmt, ...) _printf(2,3);
_protected void hex_dump(const uint8_t *tmp, size_t len, size_t llen);

_protected int vcasecmp(const struct ro_vec *v1, const struct ro_vec *v2);
_protected int vcmp(const struct ro_vec *v1, const struct ro_vec *v2);
_protected int vstrcmp(const struct ro_vec *v1, const char *str);
_protected int vcasecmp_fast(const struct ro_vec *v1, const struct ro_vec *v2);
_protected int vcmp_fast(const struct ro_vec *v1, const struct ro_vec *v2);
_protected int vstrcmp_fast(const struct ro_vec *v1, const char *str);
_protected size_t vtouint(struct ro_vec *v, unsigned int *u);

/* field types */
#define GIDX_TYPE_STRING		0x0
#define GIDX_TYPE_U8			0x1
#define GIDX_TYPE_U16			0x2
#define GIDX_TYPE_U32			0x3
#define GIDX_TYPE_U64			0x4
#define GIDX_NR_TYPES			0x5
_protected const char * const gidx_type_str(gidx_type_t);

/* schema field modes */
#define GIDX_PRIMARY_KEY		0
#define GIDX_SECONDARY_KEY		1
#define GIDX_NR_MODES			2
_protected const char * const gidx_mode_str(gidx_mode_t);

_protected int gidx_pkey_cmp(const void *, const void *);
_private void sort(void *base, size_t num, size_t size,
		  int (*cmp_func)(const void *, const void *));
#endif /* _GIDX_H */
