/*
 * This file is part of dotscara
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 */
#ifndef _FIRESTORM_OS_HEADER_INCLUDED_
#define _FIRESTORM_OS_HEADER_INCLUDED_

_private const uint8_t *map_file(int fd, size_t *len);
_private int os_errno(void);
_private const char *os_error(int);
_private const char *os_err(void);
_private const char *os_err2(const char *);

_private int fd_read(int fd, void *buf, size_t *sz, int *eof) _check_result;
_private int fd_write(int fd, const void *buf, size_t len) _check_result;
_private int fd_close(int fd);

_private int fd_block(int fd, int b);
_private int fd_coe(int fd, int coe);

/* Byte-swapping macros. Note that constant versions must actually be passed
 * literals as arguments! This is due to pre-processor badness. You have been
 * warned!
 */
#define const_bswap16(x) \
	((uint16_t)( \
		(((uint16_t)(x) & (uint16_t)0x00ffU) << 8) | \
	 	(((uint16_t)(x) & (uint16_t)0xff00U) >> 8)))

#define const_bswap32(x) \
	((uint32_t)( \
		(((uint32_t)(x) & (uint32_t)0x000000ffUL) << 24) | \
		(((uint32_t)(x) & (uint32_t)0x0000ff00UL) <<  8) | \
		(((uint32_t)(x) & (uint32_t)0x00ff0000UL) >>  8) | \
		(((uint32_t)(x) & (uint32_t)0xff000000UL) >> 24)))

static inline uint16_t _constfn sys_bswap16(uint16_t x)
{
	return const_bswap16(x);
}

static inline uint32_t _constfn sys_bswap32(uint32_t x)
{
	return const_bswap32(x);
}

#if __BYTE_ORDER == __BIG_ENDIAN
#define sys_le32(x) sys_bswap32(x)
#define sys_le16(x) sys_bswap16(x)
#define sys_be32(x) (x)
#define sys_be16(x) (x)
#define const_le32(x) const_bswap32(x)
#define const_le16(x) const_bswap16(x)
#define const_be32(x) (x)
#define const_be16(x) (x)
#elif __BYTE_ORDER == __LITTLE_ENDIAN
#define sys_le32(x) (x)
#define sys_le16(x) (x)
#define sys_be32(x) sys_bswap32(x)
#define sys_be16(x) sys_bswap16(x)
#define const_le32(x) (x)
#define const_le16(x) (x)
#define const_be32(x) const_bswap32(x)
#define const_be16(x) const_bswap16(x)
#else
#error "What in hells name?!"
#endif

#endif /* _FIRESTORM_OS_HEADER_INCLUDED_ */
